# Databricks notebook source
# DBTITLE 1,Discrepancy Index annual table
import pyspark.sql.functions as f
from pyspark.sql.functions import col,when,count


#Import UN Comtrade annual trade data
data=spark.sql('select * from prd_datascience_dianapublic.dianapublic.unsd_wits_annual')

#Table of reported imports
reported_imports=data.selectExpr('NomenCode as NomenCodeI','ReporterISO3 as ReporterISO3I','PartnerISO3 as PartnerISO3I','ProductCode as ProductCodeI','TradeValue as TradeValueI','Year as YearI','Tier as TierI','NetWeight as NetWeightI','Quantity as QuantityI','QtyToken as QtyTokenI',).where(data["TradeFlow"]==1)
  
#Table from reported exports
reported_exports=data.selectExpr('NomenCode','ReporterISO3','PartnerISO3',"ProductCode","TradeValue",'Year','Tier','NetWeight','Quantity','QtyToken').where(data["TradeFlow"]==2)
  
#Condition for matching reported imports and exports
conditions=[(reported_imports["NomenCodeI"]==reported_exports["NomenCode"]) & (reported_imports["ProductCodeI"]==reported_exports["ProductCode"]) & (reported_imports["ReporterISO3I"]==reported_exports["PartnerISO3"]) & (reported_imports["PartnerISO3I"]==reported_exports["ReporterISO3"]) & (reported_imports['YearI']==reported_exports['Year']) & (reported_imports['TierI']==reported_exports['Tier'])]

#Match reported data, handle Null values
joined_df=reported_imports.join(reported_exports,conditions,'outer')
joined_df=joined_df.select(when(joined_df['NomenCodeI'].isNull(),joined_df['NomenCode']).otherwise(joined_df['NomenCodeI']).alias('NomenCode'),when(joined_df['ReporterISO3I'].isNull(),joined_df['PartnerISO3']).otherwise(joined_df['ReporterISO3I']).alias('CountryA'),when(joined_df['PartnerISO3I'].isNull(),joined_df['ReporterISO3']).otherwise(joined_df['PartnerISO3I']).alias('CountryB'),when(joined_df['ProductCodeI'].isNull(),joined_df['ProductCode']).otherwise(joined_df['ProductCodeI']).alias("ProductCode"),when(joined_df['TradeValueI'].isNull(),0).otherwise(joined_df['TradeValueI']).alias('ImportAFromB'),when(joined_df['TradeValue'].isNull(),0).otherwise(joined_df['TradeValue']).alias('ExportBToA'),when(joined_df['YearI'].isNull(),joined_df['Year']).otherwise(joined_df['YearI']).alias('Year'),when(joined_df['TierI'].isNull(),joined_df['Tier']).otherwise(joined_df['TierI']).alias('Tier'),when(joined_df['NetWeightI'].isNull(),0).otherwise(joined_df['NetWeightI']).alias('NetWeightImportAfromB'),when(joined_df['NetWeight'].isNull(),0).otherwise(joined_df['NetWeight']).alias('NetWeightExportBToA'),when(joined_df['QuantityI'].isNull(),0).otherwise(joined_df['QuantityI']).alias('QuantityImportAfromB'),when(joined_df['Quantity'].isNull(),0).otherwise(joined_df['Quantity']).alias('QuantityExportBToA'),f.col('QtyTokenI').alias('QtyTokenImportAfromB'),f.col('QtyToken').alias('QtyTokenExportBToA'))

#Calculate the discrepancy index (DI) for the trade value and quantity of the reported data
joined_df=joined_df.select('*',(joined_df["ImportAFromB"]+joined_df["ExportBToA"]).alias("X+M"),(joined_df["ExportBToA"]-joined_df["ImportAFromB"]).alias("X-M"),(joined_df['NetWeightImportAfromB']+joined_df['NetWeightExportBToA']).alias('NWX+NWM'),(joined_df['NetWeightExportBToA']-joined_df['NetWeightImportAfromB']).alias('NWX-NWM'),(joined_df['QuantityImportAfromB']+joined_df['QuantityExportBToA']).alias('QX+QM'),(joined_df['QuantityExportBToA']-joined_df['QuantityImportAfromB']).alias('QX-QM'))
DI_df=joined_df.select('*',(joined_df['X-M']/joined_df['X+M']).alias("DI"),(joined_df['NWX-NWM']/joined_df['NWX+NWM']).alias('DI_NetWeight'),(joined_df['QX-QM']/joined_df['QX+QM']).alias('DI_Quantity'),(joined_df['QtyTokenImportAfromB']==joined_df['QtyTokenExportBToA']).alias('QtyTokenMatch'))

#Save table
table_name='prd_datascience_dianapublic.dianapublic.discrepancy_index_annual'
DI_df.write.mode("overwrite").saveAsTable(table_name)

#Display table
display(DI_df)

# COMMAND ----------

# DBTITLE 1,Country Indicators - All HS Nomenclatures
import pyspark.sql.functions as f

disc_data=spark.sql('select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual')
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
country_list=country_list.union({'SVU','ETF','DDR'})


disc_data=disc_data.where((f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list))& (f.col('NomenCode').isin(['H0','H1','H2','H3','H4','H5'])) ).withColumn('max_value',f.greatest(f.col("ImportAFromB"), f.col("ExportBToA")))
TIER=3

#Import reporting indicators
table_1=disc_data.where(f.col('Tier')==TIER).groupBy(['CountryA','Year','NomenCode']).agg(f.sum(f.col('ImportAFromB')).alias('reported_imports_sum'),f.sum(f.col('ExportBToA')).alias('reported_exports_sum'),f.count('DI').alias('num_transactions'),f.sum(f.when(f.col('DI')==-1,1).otherwise(0)).alias('num_orphan_imports'),f.sum(f.when(f.col('DI')==1,1).otherwise(0)).alias('num_lost_exports'),f.mean('DI').alias('avg_DI'),f.percentile_approx('DI',0.5).alias('median_DI'),f.percentile_approx('DI',0.25).alias('q1_DI'),f.percentile_approx('DI',0.75).alias('q3_DI'),f.sum(f.when(f.col('DI')==-1,f.col('ImportAFromB')).otherwise(0)).alias('orphan_imports_sum'),f.sum(f.when(f.col('DI')==1,f.col('ExportBToA')).otherwise(0)).alias('lost_export_sum'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<0),1).otherwise(0)).alias('num_overreported_transactions'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>0),1).otherwise(0)).alias('num_underreported_transactions'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<1),f.col('ImportAFromB')).otherwise(0)).alias('matched_imports_sum'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>-1),f.col('ExportBToA')).otherwise(0)).alias('matched_exports_sum'),f.sum(f.when(f.col('DI')>0,f.col('X-M')).otherwise(0)).alias('unaccounted_value_sum'), (f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('weighted_DI')).withColumn("cif_fob_ratio",f.col('reported_imports_sum')/f.col('reported_exports_sum')).withColumn("percetage_orphan_imports",f.col('num_orphan_imports')/f.col('num_transactions')).withColumn('percentage_lost_export',f.col('num_lost_exports')/f.col('num_transactions')).withColumn('orphan_imports_to_reported_imports_ratio',f.col('orphan_imports_sum')/f.col('reported_imports_sum')).withColumn('lost_exports_to_reported_imports_ratio',f.col('lost_export_sum')/f.col('reported_imports_sum')).withColumn('percentage_overreported_transactions',f.col('num_overreported_transactions')/f.col('num_transactions')).withColumn('percentage_underreported_transactions',f.col('num_underreported_transactions')/f.col('num_transactions')).withColumn('matched_cif_fob_ratio',f.col('matched_imports_sum')/f.col('matched_exports_sum')).withColumn('unaccounted_value_ratio',f.col('unaccounted_value_sum')/f.col('reported_imports_sum'))

#Import reporting indicators for matched transactions
table_2=disc_data.where((f.col('Tier')==TIER) & (f.col('DI')>-1) & (f.col('DI')<1)).groupBy(['CountryA','Year','NomenCode']).agg(f.mean('DI').alias('matched_avg_DI'),f.percentile_approx('DI',0.5).alias('matched_median_DI'),f.percentile_approx('DI',0.25).alias('matched_q1_DI'),f.percentile_approx('DI',0.75).alias('matched_q3_DI'),f.stddev('DI').alias('matched_std_DI'),(f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('matched_weighted_DI'))


trade_reporting_indicator_table=table_1.join(table_2,on=['CountryA','Year','NomenCode'],how='left').withColumn('flow_type',f.lit('import_reporting')).withColumnRenamed('CountryA', 'Country')


#Export reporting indicators
table_1=disc_data.where(f.col('Tier')==TIER).groupBy(['CountryB','Year','NomenCode']).agg(f.sum(f.col('ImportAFromB')).alias('reported_imports_sum'),f.sum(f.col('ExportBToA')).alias('reported_exports_sum'),f.count('DI').alias('num_transactions'),f.sum(f.when(f.col('DI')==-1,1).otherwise(0)).alias('num_orphan_imports'),f.sum(f.when(f.col('DI')==1,1).otherwise(0)).alias('num_lost_exports'),f.mean('DI').alias('avg_DI'),f.percentile_approx('DI',0.5).alias('median_DI'),f.percentile_approx('DI',0.25).alias('q1_DI'),f.percentile_approx('DI',0.75).alias('q3_DI'),f.sum(f.when(f.col('DI')==-1,f.col('ImportAFromB')).otherwise(0)).alias('orphan_imports_sum'),f.sum(f.when(f.col('DI')==1,f.col('ExportBToA')).otherwise(0)).alias('lost_export_sum'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<0),1).otherwise(0)).alias('num_underreported_transactions'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>0),1).otherwise(0)).alias('num_overreported_transactions'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<1),f.col('ImportAFromB')).otherwise(0)).alias('matched_imports_sum'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>-1),f.col('ExportBToA')).otherwise(0)).alias('matched_exports_sum'),f.sum(f.when(f.col('DI')<0,f.col('ImportAFromB')-f.col('ExportBToA')).otherwise(0)).alias('unaccounted_value_sum'), (f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('weighted_DI')).withColumn("cif_fob_ratio",f.col('reported_imports_sum')/f.col('reported_exports_sum')).withColumn("percetage_orphan_imports",f.col('num_orphan_imports')/f.col('num_transactions')).withColumn('percentage_lost_export',f.col('num_lost_exports')/f.col('num_transactions')).withColumn('orphan_imports_to_reported_imports_ratio',f.col('orphan_imports_sum')/f.col('reported_imports_sum')).withColumn('lost_exports_to_reported_imports_ratio',f.col('lost_export_sum')/f.col('reported_imports_sum')).withColumn('percentage_overreported_transactions',f.col('num_overreported_transactions')/f.col('num_transactions')).withColumn('percentage_underreported_transactions',f.col('num_underreported_transactions')/f.col('num_transactions')).withColumn('matched_cif_fob_ratio',f.col('matched_imports_sum')/f.col('matched_exports_sum')).withColumn('unaccounted_value_ratio',f.col('unaccounted_value_sum')/f.col('reported_exports_sum'))

#Export reporting indicators for matched transactions
table_2=disc_data.where((f.col('Tier')==TIER) & (f.col('DI')>-1) & (f.col('DI')<1)).groupBy(['CountryB','Year','NomenCode']).agg(f.mean('DI').alias('matched_avg_DI'),f.percentile_approx('DI',0.5).alias('matched_median_DI'),f.percentile_approx('DI',0.25).alias('matched_q1_DI'),f.percentile_approx('DI',0.75).alias('matched_q3_DI'),f.stddev('DI').alias('matched_std_DI'),(f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('matched_weighted_DI'))

trade_reporting_indicator_table_export=table_1.join(table_2,on=['CountryB','Year','NomenCode'],how='left').withColumn('flow_type',f.lit('export_reporting')).withColumnRenamed('CountryB', 'Country')

country_reporting_indicators=trade_reporting_indicator_table.union(trade_reporting_indicator_table_export).select('Country',
 'Year','NomenCode',
 'flow_type',
 'reported_imports_sum',
 'reported_exports_sum',
 'num_transactions',
 'num_orphan_imports',
 'num_lost_exports',
 'avg_DI',
 'median_DI',
 'q1_DI',
 'q3_DI',
 'orphan_imports_sum',
 'lost_export_sum',
 'num_overreported_transactions',
 'num_underreported_transactions',
 'matched_imports_sum',
 'matched_exports_sum',
 'unaccounted_value_sum',
 'weighted_DI',
 'cif_fob_ratio',
 'percetage_orphan_imports',
 'percentage_lost_export',
 'orphan_imports_to_reported_imports_ratio',
 'lost_exports_to_reported_imports_ratio',
 'percentage_overreported_transactions',
 'percentage_underreported_transactions',
 'matched_cif_fob_ratio',
 'unaccounted_value_ratio',
 'matched_avg_DI',
 'matched_median_DI',
 'matched_q1_DI',
 'matched_q3_DI',
 'matched_std_DI',
 'matched_weighted_DI',
 )

country_reporting_indicators.write.mode("overwrite").saveAsTable("prd_datascience_dianapublic.dianapublic.country_trade_reporting_indicators")


# COMMAND ----------

# DBTITLE 1,Product Indicators - All HS Nomenclatures
import pyspark.sql.functions as f
disc_data=spark.sql('select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual')

TIER=3

all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})


disc_data=disc_data.where((f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list) & (f.col('NomenCode').isin(['H0','H1','H2','H3','H4','H5'])) )).withColumn('max_value',f.greatest(f.col("ImportAFromB"), f.col("ExportBToA")))


#Trade reporting indicators 
table_1=disc_data.where(f.col('Tier')==TIER).groupBy(['ProductCode','Year','NomenCode']).agg(f.sum(f.col('ImportAFromB')).alias('reported_imports_sum'),f.sum(f.col('ExportBToA')).alias('reported_exports_sum'),f.count('DI').alias('num_transactions'),f.sum(f.when(f.col('DI')==-1,1).otherwise(0)).alias('num_orphan_imports'),f.sum(f.when(f.col('DI')==1,1).otherwise(0)).alias('num_lost_exports'),f.mean('DI').alias('avg_DI'),f.percentile_approx('DI',0.5).alias('median_DI'),f.percentile_approx('DI',0.25).alias('q1_DI'),f.percentile_approx('DI',0.75).alias('q3_DI'),f.sum(f.when(f.col('DI')==-1,f.col('ImportAFromB')).otherwise(0)).alias('orphan_imports_sum'),f.sum(f.when(f.col('DI')==1,f.col('ExportBToA')).otherwise(0)).alias('lost_export_sum'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<0),1).otherwise(0)).alias('num_import_overreported_transactions'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>0),1).otherwise(0)).alias('num_import_underreported_transactions'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<1),f.col('ImportAFromB')).otherwise(0)).alias('matched_imports_sum'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>-1),f.col('ExportBToA')).otherwise(0)).alias('matched_exports_sum'),f.sum(f.when(f.col('DI')>0,f.col('X-M')).otherwise(0)).alias('missing_imports_sum'),(f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('weighted_DI')).withColumn("cif_fob_ratio",f.col('reported_imports_sum')/f.col('reported_exports_sum')).withColumn("percetage_orphan_imports",f.col('num_orphan_imports')/f.col('num_transactions')).withColumn('percentage_lost_export',f.col('num_lost_exports')/f.col('num_transactions')).withColumn('orphan_imports_to_reported_imports_sum_ratio',f.col('orphan_imports_sum')/f.col('reported_imports_sum')).withColumn('lost_exports_to_reported_imports_sum_ratio',f.col('lost_export_sum')/f.col('reported_imports_sum')).withColumn('percentage_import_overreported_transactions',f.col('num_import_overreported_transactions')/f.col('num_transactions')).withColumn('percentage_import_underreported_transactions',f.col('num_import_underreported_transactions')/f.col('num_transactions')).withColumn('matched_cif_fob_ratio',f.col('matched_imports_sum')/f.col('matched_exports_sum')).withColumn('missing_imports_to_reported_imports_sum',f.col('missing_imports_sum')/f.col('reported_imports_sum'))

#Trade reporting indicators for matched transactions
table_2=disc_data.where((f.col('Tier')==TIER) & (f.col('DI')>-1) & (f.col('DI')<1)).groupBy(['ProductCode','Year','NomenCode']).agg(f.mean('DI').alias('matched_avg_DI'),f.percentile_approx('DI',0.5).alias('matched_median_DI'),f.percentile_approx('DI',0.25).alias('matched_q1_DI'),f.percentile_approx('DI',0.75).alias('matched_q3_DI'),f.stddev('DI').alias('matched_std_DI'),(f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('matched_weighted_DI'))

trade_reporting_indicator_table=table_1.join(table_2,on=['ProductCode','Year','NomenCode'],how='left')

trade_reporting_indicator_table.write.mode("overwrite").saveAsTable("prd_datascience_dianapublic.dianapublic.product_trade_reporting_indicators")

# COMMAND ----------

# DBTITLE 1,Country - country indicators - All HS Nomenclatures
import pyspark.sql.functions as f

disc_data=spark.sql('select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual')
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
country_list=country_list.union({'SVU','ETF','DDR'})

disc_data=disc_data.where((f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list)) & (f.col('NomenCode').isin(['H0','H1','H2','H3','H4','H5'])) ).withColumn('max_value',f.greatest(f.col("ImportAFromB"), f.col("ExportBToA")))

TIER=3

#Trade reporting indicators
table_1=disc_data.where(f.col('Tier')==TIER).groupBy(['CountryA','CountryB','Year','NomenCode']).agg(f.sum(f.col('ImportAFromB')).alias('reported_imports_sum'),f.sum(f.col('ExportBToA')).alias('reported_exports_sum'),f.count('DI').alias('num_transactions'),f.sum(f.when(f.col('DI')==-1,1).otherwise(0)).alias('num_orphan_imports'),f.sum(f.when(f.col('DI')==1,1).otherwise(0)).alias('num_lost_exports'),f.mean('DI').alias('avg_DI'),f.percentile_approx('DI',0.5).alias('median_DI'),f.percentile_approx('DI',0.25).alias('q1_DI'),f.percentile_approx('DI',0.75).alias('q3_DI'),f.sum(f.when(f.col('DI')==-1,f.col('ImportAFromB')).otherwise(0)).alias('orphan_imports_sum'),f.sum(f.when(f.col('DI')==1,f.col('ExportBToA')).otherwise(0)).alias('lost_export_sum'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<0),1).otherwise(0)).alias('num_import_overreported_transactions'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>0),1).otherwise(0)).alias('num_import_underreported_transactions'),f.sum(f.when((f.col('DI')>-1) &(f.col('DI')<1),f.col('ImportAFromB')).otherwise(0)).alias('matched_imports_sum'),f.sum(f.when((f.col('DI')<1) &(f.col('DI')>-1),f.col('ExportBToA')).otherwise(0)).alias('matched_exports_sum'),f.sum(f.when(f.col('DI')>0,f.col('X-M')).otherwise(0)).alias('export_unaccounted_value_sum'), (f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('weighted_DI')).withColumn("cif_fob_ratio",f.col('reported_imports_sum')/f.col('reported_exports_sum')).withColumn("percetage_orphan_imports",f.col('num_orphan_imports')/f.col('num_transactions')).withColumn('percentage_lost_export',f.col('num_lost_exports')/f.col('num_transactions')).withColumn('orphan_imports_to_reported_imports_ratio',f.col('orphan_imports_sum')/f.col('reported_imports_sum')).withColumn('lost_exports_to_reported_imports_ratio',f.col('lost_export_sum')/f.col('reported_imports_sum')).withColumn('percentage_overreported_transactions',f.col('num_import_overreported_transactions')/f.col('num_transactions')).withColumn('percentage_underreported_transactions',f.col('num_import_underreported_transactions')/f.col('num_transactions')).withColumn('matched_cif_fob_ratio',f.col('matched_imports_sum')/f.col('matched_exports_sum')).withColumn('export_unaccounted_value_ratio',f.col('export_unaccounted_value_sum')/f.col('reported_imports_sum'))

#Trade reporting indicators for matched transactions
table_2=disc_data.where((f.col('Tier')==TIER) & (f.col('DI')>-1) & (f.col('DI')<1)).groupBy(['CountryA','CountryB','Year','NomenCode']).agg(f.mean('DI').alias('matched_avg_DI'),f.percentile_approx('DI',0.5).alias('matched_median_DI'),f.percentile_approx('DI',0.25).alias('matched_q1_DI'),f.percentile_approx('DI',0.75).alias('matched_q3_DI'),f.stddev('DI').alias('matched_std_DI'),(f.sum(f.col('max_value') * f.col("DI"))/ f.sum(f.col('max_value')) ).alias('matched_weighted_DI'))


trade_reporting_indicator_table=table_1.join(table_2,on=['CountryA','CountryB','Year','NomenCode'],how='left')


trade_reporting_indicator_table.write.mode("overwrite").saveAsTable("prd_datascience_dianapublic.dianapublic.country_country_trade_reporting_indicators")

# COMMAND ----------

