# Databricks notebook source
import pyspark.sql.functions as f
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.express as px
import pandas as pd
import numpy as np

#Import discrepancy index dataset
discrepancy_data=spark.sql("select * from prd_datascience_dianapublic.diana.discrepancy_index_annual")

#Dual records
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})

discrepancy_data= discrepancy_data.where(( (f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list)) ))

discrepancy_data=discrepancy_data.withColumn('W_E',f.col('DI')*f.col('ExportBToA')).withColumn('W_I',f.col('DI')*f.col('ImportAFromB'))
discrepancy_data = discrepancy_data.withColumn("W", f.when(f.col("ExportBToA") >= f.col("ImportAFromB"), f.col("ExportBToA")).otherwise(f.col("ImportAFromB")) )
discrepancy_data = discrepancy_data.withColumn("WDI", f.col('W')*f.col('DI') )

group1= discrepancy_data.groupby(['NomenCode','Year','Tier']).agg(f.count('Tier').alias('total'),f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , 1).otherwise(0)).alias("matched"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ImportAFromB')).otherwise(0)).alias("matched_import_sum"), f.sum(f.when( ((f.col('DI')>-1) & (f.col('DI')<1)) , f.col('ExportBToA')).otherwise(0)).alias("matched_export_sum"),f.sum('ImportAFromB').alias('total_imports'),f.sum('ExportBToA').alias('total_exports'),f.sum('W_E').alias('W_E_sum'),f.sum('W_I').alias('W_I_sum'), f.sum('W').alias('total_W'),f.sum('WDI').alias('total_WDI')).withColumn("Dual recording",f.col('matched')/f.col('total')).withColumn("Export value with dual recording",f.col('matched_export_sum')/f.col('total_exports')).withColumn("Import value with dual recording",f.col('matched_import_sum')/f.col('total_imports')).withColumn('Weighted_di_imports',f.col('W_I_sum')/f.col('total_imports')).withColumn('Weighted_di_exports',f.col('W_E_sum')/f.col('total_exports')).orderBy(['NomenCode','Year','Tier']).withColumn('Weighted DI',f.col('total_WDI')/f.col('total_W'))
group1_pd=group1.toPandas()

# COMMAND ----------

def set_layout(fig):
  # Update layout
  
  fig.update_layout(
      title_x=0.5,
      yaxis=dict(range=[0, 1.01],title_standoff=20),
      yaxis2=dict(range=[0, 1.01],title_standoff=20),
      xaxis=dict(title_standoff=20),
      xaxis2=dict(title_standoff=20)
  )

  fig.update_layout(
      xaxis_title="<b>Year</b>",
      xaxis_title_font=dict(size=24),
      yaxis_title="<b>Percentage</b>",
      yaxis_title_font=dict(size=24),
      xaxis2_title="<b>Year</b>",
      xaxis2_title_font=dict(size=23),
      yaxis2_title="<b>Percentage</b>",
      yaxis2_title_font=dict(size=23),
      yaxis_tickformat='.0%',  
      yaxis2_tickformat='.0%',
      xaxis_tickfont=dict(size=20),
      yaxis_tickfont=dict(size=20),
      xaxis2_tickfont=dict(size=20),
      yaxis2_tickfont=dict(size=20),
      width=1920,  
      height=1080,  
      plot_bgcolor='white',  
      xaxis_gridcolor='lightgray',  
      yaxis_gridcolor='lightgray',  
      xaxis2_gridcolor='lightgray',
      yaxis2_gridcolor='lightgray',
  )

  fig.update_layout(
      legend=dict(
          orientation="h",  
          xanchor="center", 
          yanchor="bottom", 
          x=0.49,  
          y=-0.22,  
          font=dict(
              size=24  
          )
      ),
  )

  for i in range(len(fig.layout.annotations)):
      fig.layout.annotations[i].font.size = 23
      fig.layout.annotations[i].y += 0.07
  
  return fig


# COMMAND ----------

# DBTITLE 1,Figure 5

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Standard International Trade Classification (SITC Rev.1)', ' The Harmonized System (HS 1988) ']
nomenclature_codes = [('S1', 1), ('H0', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()


# COMMAND ----------

# DBTITLE 1,Figure 31 - Part 1
#HS0, HS1

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Harmonized System (HS0)', ' The Harmonized System (HS1)']
nomenclature_codes = [('H0', 1), ('H1', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 31 - Part 2
#HS2 HS3

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Harmonized System (HS2)', ' The Harmonized System (HS3)']
nomenclature_codes = [('H2', 1), ('H3', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 31 - Part 3
#HS4, HS5

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Harmonized System (HS4)', ' The Harmonized System (HS5)']
nomenclature_codes = [('H4', 1), ('H5', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 32 - Part 1
#SITC 1, SITC 2

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Standard International Trade Classification (SITC Rev.1)', 'The Standard International Trade Classification (SITC Rev.2)']
nomenclature_codes = [('S1', 1), ('S2', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 33 - Part 2
#SITC 1, SITC 2

import plotly.express as px
from plotly.subplots import make_subplots
import plotly.graph_objects as go

subplot_titles=['The Standard International Trade Classification (SITC Rev.3)', 'The Standard International Trade Classification (SITC Rev.4)']
nomenclature_codes = [('S3', 1), ('S4', 2)]  


line_thickness = 4.6 
color_palette = [px.colors.sequential.Teal[3], 'steelblue', 'indianred']

df = group1_pd[group1_pd['Year']<=2022]

subplot_titles=['<b>' + subplot_title +'</b>' for subplot_title in subplot_titles]

fig = make_subplots(rows=1, cols=2, subplot_titles=subplot_titles)

columns = ['Dual recording', 'Export value with dual recording', 'Import value with dual recording']

for nomen_code, legend_flag in nomenclature_codes:
    hs0_data = df[(df['NomenCode'] == nomen_code) & (df['Tier'] == '0')]
    print(nomen_code)
    for i, column in enumerate(columns):
        dash_style = 'dash' if 'dual recording' in column else 'solid'
        legend_show = True if legend_flag == 1 else False  # Show legend only for the first column
        fig.add_trace(
            go.Scatter(x=hs0_data['Year'], y=hs0_data[column], mode='lines', name='<b>'+column+'</b>',
                       line=dict(color=color_palette[i], width=line_thickness, dash=dash_style), showlegend=legend_show),
            row=1, col=legend_flag
        )
        print(column)
        print(hs0_data[column])


fig=set_layout(fig)

fig.show()
