# Databricks notebook source
import pyspark.sql.functions as f
import plotly.express as px
import pandas as pd
import plotly.graph_objects as go
import numpy as np
from pyspark.sql.types import IntegerType

Year=2017
flow_type='import_reporting'
nomen_code='H0'

#Country list
all_countries_list=spark.sql("select * from prd_datascience_dianapublic.diana.income_region_country_list")
country_list=set([row[0] for row in all_countries_list.select('ISO3I').where(f.col('ISO3I').isNotNull()).collect()])
country_list=country_list.union(set([row[0] for row in all_countries_list.select('ISO3').where(f.col('ISO3').isNotNull()).collect()]))
#Adding SVU, ETF and DDR
country_list=country_list.union({'SVU','ETF','DDR'})

trade_reporting_indicators=spark.sql('select * from prd_datascience_dianapublic.dianapublic.country_trade_reporting_indicators')
trade_reporting_indicators=trade_reporting_indicators.where((f.col('NomenCode')=='H0') & (f.col('flow_type')=='import_reporting') & (f.col('Year')<2022))

viz_df=trade_reporting_indicators.where( (f.col('NomenCode')==nomen_code) & (f.col('flow_type')==flow_type) & (f.col('Year')==Year) ).select('Country','percetage_orphan_imports','percentage_lost_export').toPandas()
income_region_df=all_countries_list.select(f.col('ISO3I').alias('Country'),f.col('2017').alias("Income"),f.col('RegionName')).toPandas()

viz_df['matched']=1-(viz_df['percetage_orphan_imports']+viz_df['percentage_lost_export'])
viz_df=viz_df.merge(income_region_df,on='Country')
viz_df.columns=['Country','Orphan imports','Lost exports','Matched','Income','Region Name']


#Reporting tables in Appendix
discrepancy_data=spark.sql('select * from prd_datascience_dianapublic.dianapublic.discrepancy_index_annual')
discrepancy_data=discrepancy_data.where( (f.col('NomenCode').isin(['H0','H1','H2','H3','H4','H5'])) & (f.col('Year')<=2019) & (f.col('CountryA').isin(country_list)) & (f.col('CountryB').isin(country_list)) )
discrepancy_data.groupby(['NomenCode','Year'])

# Define a UDF to count unique elements in the union of two lists
def count_unique_union(list1, list2):
    return len(set(list1 + list2))

# Register UDF
count_unique_union_udf = f.udf(count_unique_union, IntegerType())

import_reporting = discrepancy_data.groupBy('NomenCode', 'Year')\
    .agg(
        # Count distinct 'CountryA' where 'ImportAFromB' > 0
        f.countDistinct(f.when(discrepancy_data['ImportAFromB'] > 0, discrepancy_data['CountryA'])).alias('Number of import reporters'),
        # Count distinct 'CountryA'
        f.countDistinct('CountryA').alias('Number of export partners'),
        f.collect_set('CountryA').alias('unique_CountryA'),
        f.collect_set('CountryB').alias('unique_CountryB')
       
    )
import_reporting = import_reporting.withColumn('Total Number of Countries', count_unique_union_udf('unique_CountryA', 'unique_CountryB'))\
                                       .drop('unique_CountryA', 'unique_CountryB')

import_reporting =import_reporting.withColumn('Import reporters as a percentage of all countries',f.col('Number of import reporters')/f.col('Total Number of Countries'))
import_reporting_pd=import_reporting.toPandas()
import_reporting_pd['Import reporters as a percentage of all countries']=np.round(import_reporting_pd['Import reporters as a percentage of all countries'],3)

export_reporting = discrepancy_data.groupBy('NomenCode', 'Year')\
    .agg(
        # Count distinct 'CountryA' where 'ImportAFromB' > 0
        f.countDistinct(f.when(discrepancy_data['ExportBToA'] > 0, discrepancy_data['CountryB'])).alias('Number of export reporters'),
        # Count distinct 'CountryA'
        f.countDistinct('CountryB').alias('Number of import partners'),
        f.collect_set('CountryA').alias('unique_CountryA'),
        f.collect_set('CountryB').alias('unique_CountryB')
       
    )
export_reporting = export_reporting.withColumn('Total Number of Countries', count_unique_union_udf('unique_CountryA', 'unique_CountryB'))\
                                       .drop('unique_CountryA', 'unique_CountryB')

export_reporting =export_reporting.withColumn('Export reporters as a percentage of all countries',f.col('Number of export reporters')/f.col('Total Number of Countries'))
export_reporting_pd=export_reporting.toPandas()
export_reporting_pd['Export reporters as a percentage of all countries']=np.round(export_reporting_pd['Export reporters as a percentage of all countries'],3)

# COMMAND ----------

# DBTITLE 1,Figure 15

fig = px.bar(viz_df[viz_df['Income']=='L'], x="Country", y=['Lost exports','Matched','Orphan imports'], title="Low Income")
# Update layout to adjust height, width, remove axis titles, and increase font sizes
fig.update_layout(
    height=600,  
    width=900,   
    xaxis_title=None,  
    yaxis_title=None,  
    title_font=dict(size=24),  
    xaxis=dict(tickfont=dict(size=14)),  
    yaxis=dict(tickfont=dict(size=14)), 
    legend_title_font_size=16  
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 16
fig = px.bar(viz_df[viz_df['Income']=='LM'], x="Country", y=['Lost exports','Matched','Orphan imports'], title="Lower-middle Income")
# Update layout to adjust height, width, remove axis titles, and increase font sizes
fig.update_layout(
    height=600,  
    width=900,   
    xaxis_title=None,  
    yaxis_title=None,  
    title_font=dict(size=24),  
    xaxis=dict(tickfont=dict(size=14)),  
    yaxis=dict(tickfont=dict(size=14)), 
    legend_title_font_size=16  
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 17
fig = px.bar(viz_df[viz_df['Income']=='H'], x="Country", y=['Lost exports','Matched','Orphan imports'], title="High Income")
# Update layout to adjust height, width, remove axis titles, and increase font sizes
fig.update_layout(
    height=600,  
    width=900,   
    xaxis_title=None,  
    yaxis_title=None,  
    title_font=dict(size=24),  
    xaxis=dict(tickfont=dict(size=14)),  
    yaxis=dict(tickfont=dict(size=14)), 
    legend_title_font_size=16  
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 18
fig = px.bar(viz_df[viz_df['Region Name']=='Sub-Saharan Africa'], x="Country", y=['Lost exports','Matched','Orphan imports'], title="Sub-Saharan Africa")
# Update layout to adjust height, width, remove axis titles, and increase font sizes
fig.update_layout(
    height=600,  
    width=900,   
    xaxis_title=None,  
    yaxis_title=None,  
    title_font=dict(size=24),  
    xaxis=dict(tickfont=dict(size=14)),  
    yaxis=dict(tickfont=dict(size=14)), 
    legend_title_font_size=16  
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 19
fig = px.bar(viz_df[viz_df['Region Name']=='Europe & Central Asia'], x="Country", y=['Lost exports','Matched','Orphan imports'], title="Europe & Central Asia")
# Update layout to adjust height, width, remove axis titles, and increase font sizes
fig.update_layout(
    height=600,  
    width=900,   
    xaxis_title=None,  
    yaxis_title=None,  
    title_font=dict(size=24),  
    xaxis=dict(tickfont=dict(size=14)),  
    yaxis=dict(tickfont=dict(size=14)), 
    legend_title_font_size=16  
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 3


nomen_code='H0'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=1200,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 4

nomen_code='H1'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=800,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 5
nomen_code='H2'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=600,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 6
nomen_code='H3'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=600,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 7
nomen_code='H4'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=400,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 8
nomen_code='H5'

fig = go.Figure(data=[go.Table(
    header=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=import_reporting_pd[import_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=400,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 9
nomen_code='H0'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=1200,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 10
nomen_code='H1'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=800,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 11
nomen_code='H2'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=600,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 12
nomen_code='H3'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=600,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 13
nomen_code='H4'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=400,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()

# COMMAND ----------

# DBTITLE 1,Table 14
nomen_code='H5'

fig = go.Figure(data=[go.Table(
    header=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].columns[1:], fill_color='rgba(70, 130, 180, 0.7)', align='center',line_color='black',font=dict(size=12, color='black')),
    cells=dict(values=export_reporting_pd[export_reporting_pd['NomenCode']==nomen_code].values.T[1:,:],
               align='center',line_color='black',font=dict(size=12, color='black'),height=30,  # Adjust cell height for more padding
        ))
])
fig.update_layout(
    width=1200,
    height=400,
    margin=dict(l=2, r=2, t=2, b=2)  # Set left, right, top, and bottom margins to 0
)
fig.show()