# Databricks notebook source
import pyspark.sql.functions as f
from scipy.stats import mannwhitneyu,ttest_ind
import numpy as np
import statsmodels.api as sm
from statsmodels.sandbox.regression.predstd import wls_prediction_std
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go
from plotly.subplots import make_subplots

#Data
#Trade reporting indicators
country_pair_reporting_indicators_data=spark.sql('select * from prd_datascience_dianapublic.diana.country_country_trade_reporting_indicators ')
country_reporting_indicators_data=spark.sql('select * from prd_datascience_dianapublic.diana.country_trade_reporting_indicators ')

#CEPII Dist data (1948-2021)
cepii_gravity_data=spark.sql('select * from prd_datascience_dianapublic.diana.cepii_gravity_202211')

#GDP Data WDI (1960-2022) (Indicator: GDP per capita, PPP (constant 2017 international $))
gdp_data=spark.sql('select * from prd_datascience_dianapublic.diana.wdi_gdp')

#Statistical capacity (2004-2019)
spi_data=spark.sql('select * from prd_datascience_dianapublic.default.spi_index')


NomenCode='H0'
YEAR=2017

cepii_gravity_subset=cepii_gravity_data.where(f.col('year')==YEAR).select(f.col('iso3_o').alias('CountryA') ,f.col('iso3_d').alias('CountryB'),f.col('contig'),f.col('dist'),f.col('fta_wto')).na.drop()
cepii_gravity_subset = cepii_gravity_subset.replace(['ROU','COD','MNE','TLS'], ['ROM','ZAR','MNT','TMP'], subset=['CountryA'])
cepii_gravity_subset = cepii_gravity_subset.replace(['ROU','COD','MNE','TLS'], ['ROM','ZAR','MNT','TMP'], subset=['CountryB'])

rez_1=country_pair_reporting_indicators_data.where( (f.col('Year')==YEAR) & (f.col('NomenCode')==NomenCode)).withColumn("max_value", f.when(f.col("reported_imports_sum") >= f.col("reported_exports_sum"), f.col("reported_imports_sum")).otherwise(f.col("reported_exports_sum"))).join(cepii_gravity_subset,on=['CountryA','CountryB'],how='left')

rez_1_pd=rez_1.toPandas()
rez_1_pd['log_dist']=np.log(rez_1_pd['dist'])


flow_type='import_reporting'

spi_data_subset = spi_data.where(f.col('date') == YEAR).select(
    f.col('iso3c').alias('Country'),
    f.col('`SPI.INDEX.PIL1`').cast('float').alias('spi_index_pil1'),
    f.col('`SPI.INDEX.PIL2`').cast('float').alias('spi_index_pil2'),
    f.col('`SPI.INDEX.PIL3`').cast('float').alias('spi_index_pil3'),
    f.col('`SPI.INDEX.PIL4`').cast('float').alias('spi_index_pil4'),
    f.col('`SPI.INDEX.PIL5`').cast('float').alias('spi_index_pil5'),
    f.col('`SPI.INDEX`').cast('float').alias('spi_index')
)
spi_data_subset = spi_data_subset.replace(['ROU','COD','MNE','TLS'], ['ROM','ZAR','MNT','TMP'], subset=['Country'])

rez_2=country_reporting_indicators_data.where( (f.col('Year')==YEAR) & (f.col('NomenCode')==NomenCode) & (f.col('flow_type')=='import_reporting')).withColumn("max_value", f.when(f.col("reported_imports_sum") >= f.col("reported_exports_sum"), f.col("reported_imports_sum")).otherwise(f.col("reported_exports_sum"))).join(spi_data_subset,on=['Country'],how='left')

rez_2_pd=rez_2.toPandas()

import_reporting=country_reporting_indicators_data.where( (f.col('Year')==YEAR) & (f.col('NomenCode')==NomenCode) & (f.col('flow_type')=='import_reporting')).orderBy('Country').toPandas()

export_reporting=country_reporting_indicators_data.where( (f.col('Year')==YEAR) & (f.col('NomenCode')==NomenCode) & (f.col('flow_type')=='export_reporting')).orderBy('Country').toPandas()


# COMMAND ----------

# DBTITLE 1,Figure 28.A
colors = ['rgba(31, 119, 180, 0.8)','rgba(205, 92, 92, 0.8)'] 

#contig, fta_wto
dummies=['contig','fta_wto']
Indicator='percetage_orphan_imports'
viz_output=[]

test_statistics=[]
n_obs=rez_1_pd[Indicator].count()
print(f"Number of observations: {n_obs}")
for dummy in dummies:
  print(dummy)
  data_group_a=rez_1_pd[rez_1_pd[dummy]==0][Indicator]
  data_group_b=rez_1_pd[rez_1_pd[dummy]==1][Indicator]

  u_stat, p_val = ttest_ind(data_group_a, data_group_b)
  print(f'T-Test: t={u_stat}, p={p_val}')
  print(p_val<0.01)
  test_statistics.append([u_stat,p_val])


  mean_group_a = data_group_a.mean()
  mean_group_b = data_group_b.mean()

  viz_output.append([dummy,mean_group_a,mean_group_b])


# Creating the bar graph
fig = go.Figure()
x_name='Contiguity'+ ' ('+str(round(test_statistics[0][0],3))+"***)"
fig.add_trace(go.Bar(x=[x_name], y=[viz_output[0][1]], name='No',marker_color=colors[0]))
fig.add_trace(go.Bar(x=[x_name], y=[viz_output[0][2]], name='Yes',marker_color=colors[1]))

x1_name='RTA'+ ' ('+str(round(test_statistics[1][0],3))+"***)"
fig.add_trace(go.Bar(x=[x1_name], y=[viz_output[1][1]], name='0',showlegend=False,marker_color=colors[0]))
fig.add_trace(go.Bar(x=[x1_name], y=[viz_output[1][2]], name='1',showlegend=False,marker_color=colors[1]))

# Adding titles and labels
fig.update_layout(xaxis = {
        'tickmode': 'array',
        'tickvals': [-0.2, 1.21],

    },
                  barmode='group',plot_bgcolor='white',
    width=900, 
    height=700,
    yaxis=dict(range=[0, 0.75]),
    legend=dict(font=dict(size=16)))
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Percentage of Orphan Imports',title_font=dict(size=22))  

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 28.B
from scipy.stats import mannwhitneyu
import plotly.graph_objects as go
colors = ['rgba(31, 119, 180, 0.8)','rgba(205, 92, 92, 0.8)'] 

#contig, fta_wto
dummies=['contig','fta_wto']
Indicator='weighted_DI'
viz_output=[]

test_statistics=[]
n_obs=rez_1_pd[Indicator].count()
print(f"Number of observations: {n_obs}")
for dummy in dummies:
  print(dummy)
  data_group_a=rez_1_pd[rez_1_pd[dummy]==0][Indicator]
  data_group_b=rez_1_pd[rez_1_pd[dummy]==1][Indicator]

  u_stat, p_val = ttest_ind(data_group_a, data_group_b)
  print(f'T-Test: t={u_stat}, p={p_val}')
  print(p_val<0.01)
  test_statistics.append([u_stat,p_val])

  mean_group_a = data_group_a.mean()
  mean_group_b = data_group_b.mean()

  viz_output.append([dummy,mean_group_a,mean_group_b])


# Creating the bar graph
fig = go.Figure()
x_name='Contiguity'+ ' ('+str(round(test_statistics[0][0],3))+"***)"
fig.add_trace(go.Bar(x=[x_name], y=[viz_output[0][1]], name='No',marker_color=colors[0]))
fig.add_trace(go.Bar(x=[x_name], y=[viz_output[0][2]], name='Yes',marker_color=colors[1]))

x1_name='RTA'+ ' ('+str(round(test_statistics[1][0],3))+"***)"
fig.add_trace(go.Bar(x=[x1_name], y=[viz_output[1][1]], name='0',showlegend=False,marker_color=colors[0]))
fig.add_trace(go.Bar(x=[x1_name], y=[viz_output[1][2]], name='1',showlegend=False,marker_color=colors[1]))

fig.add_trace(go.Scatter(x=[x_name, x1_name], y=[0, 0], mode='lines', 
                         line=dict(color='lightgray', width=2, dash='dash'), 
                         showlegend=False))

# Adding titles and labels
fig.update_layout(xaxis = {
        'tickmode': 'array',
        'tickvals': [-0.15, 1.21],

    },
                  barmode='group',plot_bgcolor='white',
    width=900, 
    height=700,
    yaxis=dict(range=[-0.35, 0.2]),
    legend=dict(font=dict(size=16)))
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Weighted Discepancy Index',title_font=dict(size=22))  

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 28.C
Indicator='percetage_orphan_imports'
dist_plot_data=rez_1_pd[['log_dist',Indicator]]
dist_plot_data

#Excluding very short distances, sorting because of plotly plot requirements
X = rez_1_pd[rez_1_pd['log_dist']>=5].sort_values('log_dist')['log_dist'].values
Y = rez_1_pd[rez_1_pd['log_dist']>=5].sort_values('log_dist')['percetage_orphan_imports'].values

# Removing NaN values
is_not_nan = ~np.isnan(X) & ~np.isnan(Y)
X = X[is_not_nan]
Y = Y[is_not_nan]

print(f'Number of observations {len(X)}')

X_with_const = sm.add_constant(X)

#OLS model
model = sm.OLS(Y, X_with_const).fit()

#Confidence intervals
predictions_summary_frame = model.get_prediction(X_with_const).summary_frame(alpha=0.05)
mean_predictions = predictions_summary_frame['mean']
mean_ci_lower = predictions_summary_frame['mean_ci_lower']
mean_ci_upper = predictions_summary_frame['mean_ci_upper']

#OWESS smoothing for comparison
lowess = sm.nonparametric.lowess(Y, X, frac=0.35)


fig = go.Figure()


fig.add_trace(go.Scatter(x=X, y=mean_ci_lower, mode='lines', # Increased opacity
                         line=dict(width=5,color='lightgray'), name='95% Confidence Interval', showlegend=True))


fig.add_trace(go.Scatter(x=X, y=mean_ci_upper, mode='lines', fill='tonexty',
                         fillcolor='lightgray', line=dict(width=5,color='lightgray'), showlegend=False))


# OLS regression line
fig.add_trace(go.Scatter(x=X, y=mean_predictions, mode='lines', name='OLS Fit',line=dict(color='steelblue',width=3)))

# LOWESS smoothed line
fig.add_trace(go.Scatter(x=lowess[:, 0], y=lowess[:, 1], mode='lines', name='LOWESS',line=dict(dash='dash',color='rgba(205, 92, 92, 0.8)',width=3)))

# Update the layout
fig.update_layout(
    xaxis_title='Log Distance between most populated cities (km)',
    yaxis_title='Percentage of Orphan Imports',
    yaxis=dict(range=[0, 1],showgrid=True,gridcolor='lightgray'),
    plot_bgcolor='white',
    legend=dict(yanchor="top", y=0.99, xanchor="left", x=0.01,font=dict(size=18)),
    xaxis=dict(showgrid=True,gridcolor='lightgray',range=[5,10]),
    height=700,
    width=1100
)
r_squared = model.rsquared
print(f'R^2: {r_squared}')
# For individual coefficients
p_values = model.pvalues
print(f'P-values for each coefficient:\n{p_values}')

# For the overall model significance (F-statistic)
f_pvalue = model.f_pvalue
print(f'P-value from F-statistic (overall model significance): {f_pvalue}')
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20),title_font=dict(size=22))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Percentage of Orphan Imports',title_font=dict(size=22))  
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 29.A
spi_indicator='spi_index_pil3'
trade_reporting_indicator='weighted_DI'


X = rez_2_pd.sort_values(spi_indicator)[spi_indicator].values
Y = rez_2_pd.sort_values(spi_indicator)[trade_reporting_indicator].values

# Removing NaN values
is_not_nan = ~np.isnan(X) & ~np.isnan(Y)
X = X[is_not_nan]
Y = Y[is_not_nan]

print(f'Number of observations: {len(X)}')

X_with_const = sm.add_constant(X)

#OLS model
model = sm.OLS(Y, X_with_const).fit()

#Confidence intervals
predictions_summary_frame = model.get_prediction(X_with_const).summary_frame(alpha=0.05)
mean_predictions = predictions_summary_frame['mean']
mean_ci_lower = predictions_summary_frame['mean_ci_lower']
mean_ci_upper = predictions_summary_frame['mean_ci_upper']

#LOWESS smoothing for comparison
lowess = sm.nonparametric.lowess(Y, X, frac=0.35)


fig = go.Figure()

fig.add_trace(go.Scatter(x=X, y=mean_ci_lower, mode='lines', # Increased opacity
                         line=dict(width=5,color='lightgray'), name='95% Confidence Interval', showlegend=True))

fig.add_trace(go.Scatter(x=X, y=mean_ci_upper, mode='lines', fill='tonexty',
                         fillcolor='lightgray', line=dict(width=5,color='lightgray'), showlegend=False))


# OLS regression line
fig.add_trace(go.Scatter(x=X, y=mean_predictions, mode='lines', name='OLS Fit',line=dict(color='steelblue',width=4)))

# LOWESS smoothed line
fig.add_trace(go.Scatter(x=lowess[:, 0], y=lowess[:, 1], mode='lines', name='LOWESS',line=dict(dash='dash',color='rgba(205, 92, 92, 0.8)',width=4)))

# Update the layout
fig.update_layout(
    xaxis_title='Statistical Performance Indicator (Pillar 3)',
    yaxis_title='Median DI',
    yaxis=dict(range=[-1, 1],showgrid=True,gridcolor='lightgray'),
    plot_bgcolor='white',
    legend=dict(yanchor="top", y=0.99, xanchor="left", x=0.68,font=dict(size=18)),
    xaxis=dict(showgrid=True,gridcolor='lightgray'),
    height=700,
    width=1100
)
r_squared = model.rsquared
print(f'R^2: {r_squared}')
# For individual coefficients
p_values = model.pvalues
print(f'P-values for each coefficient:\n{p_values}')

# For the overall model significance (F-statistic)
f_pvalue = model.f_pvalue
print(f'P-value from F-statistic (overall model significance): {f_pvalue}')
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20),title_font=dict(size=22))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Median Discrepancy Index',title_font=dict(size=22)) 
fig.add_hline(y=0, line_dash="dash", line_color="lightgray", line_width=2)
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 29.B
spi_indicator='spi_index'
trade_reporting_indicator='percetage_orphan_imports'

#Exluding non-reporters and sorting for plotly plot
X = rez_2_pd[rez_2_pd['percetage_orphan_imports']>0].sort_values(spi_indicator)[spi_indicator].values
Y = rez_2_pd[rez_2_pd['percetage_orphan_imports']>0].sort_values(spi_indicator)[trade_reporting_indicator].values

#Removing NaN values
is_not_nan = ~np.isnan(X) & ~np.isnan(Y)
X = X[is_not_nan]
Y = Y[is_not_nan]

print(f'Number of observations: {len(X)}')

X_with_const = sm.add_constant(X)

#OLS model
model = sm.OLS(Y, X_with_const).fit()

#Confidence intervals
predictions_summary_frame = model.get_prediction(X_with_const).summary_frame(alpha=0.05)
mean_predictions = predictions_summary_frame['mean']
mean_ci_lower = predictions_summary_frame['mean_ci_lower']
mean_ci_upper = predictions_summary_frame['mean_ci_upper']

#LOWESS smoothing for comparison
lowess = sm.nonparametric.lowess(Y, X, frac=0.35)


fig = go.Figure()

fig.add_trace(go.Scatter(x=X, y=mean_ci_lower, mode='lines', # Increased opacity
                         line=dict(width=5,color='lightgray'), name='95% Confidence Interval', showlegend=True))

# 95% confidence interval for the fitted line - Lower bound first
fig.add_trace(go.Scatter(x=X, y=mean_ci_upper, mode='lines', fill='tonexty',
                         fillcolor='lightgray', line=dict(width=5,color='lightgray'), showlegend=False))



# OLS regression line
fig.add_trace(go.Scatter(x=X, y=mean_predictions, mode='lines', name='OLS Fit',line=dict(color='steelblue',width=4)))

# LOWESS smoothed line
fig.add_trace(go.Scatter(x=lowess[:, 0], y=lowess[:, 1], mode='lines', name='LOWESS',line=dict(dash='dash',color='rgba(205, 92, 92, 0.8)',width=4)))

# Update the layout
fig.update_layout(
    xaxis_title='Statistical Performance Indicator',
    yaxis_title='Percentage of Orphan Imports',
    yaxis=dict(range=[0, 1],showgrid=True,gridcolor='lightgray'),
    plot_bgcolor='white',
    legend=dict(yanchor="top", y=0.99, xanchor="left", x=0.01,font=dict(size=18)),
    xaxis=dict(showgrid=True,gridcolor='lightgray'),
    height=700,
    width=1100
)
r_squared = model.rsquared
print(f'R^2: {r_squared}')
# For individual coefficients
p_values = model.pvalues
print(f'P-values for each coefficient:\n{p_values}')

# For the overall model significance (F-statistic)
f_pvalue = model.f_pvalue
print(f'P-value from F-statistic (overall model significance): {f_pvalue}')
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20),title_font=dict(size=22))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Percentage of Orphan Imports',title_font=dict(size=22))  
fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 30
Indicator_1='weighted_DI'
Indicator_2='weighted_DI'

#Sorting because of plotly plot
merged_table=import_reporting.sort_values(Indicator_1)[['Country',Indicator_1]].merge(export_reporting[['Country',Indicator_2]],on=['Country'])
if Indicator_1==Indicator_2:
  X = merged_table[Indicator_1+'_x'].values
  Y = merged_table[Indicator_1+'_y'].values
else:
  X = merged_table[Indicator_1].values
  Y = merged_table[Indicator_2].values

#Removing NaN values
is_not_nan = ~np.isnan(X) & ~np.isnan(Y)
X = X[is_not_nan]
Y = Y[is_not_nan]

print(f'Number of observations: {len(X)}')

X_with_const = sm.add_constant(X)

#OLS model
model = sm.OLS(Y, X_with_const).fit()

#Confidence intervals
predictions_summary_frame = model.get_prediction(X_with_const).summary_frame(alpha=0.05)
mean_predictions = predictions_summary_frame['mean']
mean_ci_lower = predictions_summary_frame['mean_ci_lower']
mean_ci_upper = predictions_summary_frame['mean_ci_upper']

#LOWESS smoothing for comparison
lowess = sm.nonparametric.lowess(Y, X, frac=0.35)


fig = go.Figure()

fig.add_trace(go.Scatter(x=X, y=mean_ci_lower, mode='lines', # Increased opacity
                         line=dict(width=5,color='lightgray'), name='95% Confidence Interval', showlegend=True))

# 95% confidence interval for the fitted line - Lower bound first
fig.add_trace(go.Scatter(x=X, y=mean_ci_upper, mode='lines', fill='tonexty',
                         fillcolor='lightgray', line=dict(width=5,color='lightgray'), showlegend=False))



# OLS regression line
fig.add_trace(go.Scatter(x=X, y=mean_predictions, mode='lines', name='OLS Fit',line=dict(color='steelblue',width=4)))

# LOWESS smoothed line
fig.add_trace(go.Scatter(x=lowess[:, 0], y=lowess[:, 1], mode='lines', name='LOWESS',line=dict(dash='dash',color='rgba(205, 92, 92, 0.8)',width=4)))

# Update the layout
fig.update_layout(
    xaxis_title='Import Reporting (Weighted DI)',
    yaxis_title='Export Reporting (Weighted DI)',
    yaxis=dict(showgrid=True,gridcolor='lightgray'),
    plot_bgcolor='white',
    legend=dict(yanchor="top", y=0.99, xanchor="left", x=0.1,font=dict(size=18)),
    xaxis=dict(showgrid=True,gridcolor='lightgray'),
    height=700,
    width=1100
)
r_squared = model.rsquared
print(f'R^2: {r_squared}')
# For individual coefficients
p_values = model.pvalues
print(f'P-values for each coefficient:\n{p_values}')

# For the overall model significance (F-statistic)
f_pvalue = model.f_pvalue
print(f'P-value from F-statistic (overall model significance): {f_pvalue}')
fig.update_xaxes(showline=True, linewidth=2, linecolor='black',tickfont=dict(size=20),title_font=dict(size=22))
fig.update_yaxes(showline=True, linewidth=2, linecolor='black', gridcolor='lightgray',tickfont=dict(size=18),title_text='Export Reporting (Weighted DI)',title_font=dict(size=22))  

fig.show()

# COMMAND ----------

# DBTITLE 1,Figure 33
#CIF and FOB Trade Balance calculation
temp_table_1 = country_reporting_indicators_data \
    .where((f.col('NomenCode') == NomenCode) & (f.col('flow_type')=='import_reporting') & (f.col('Year')<=2021)) \
    .select(
        f.col('Country').alias('Country_1'),
        f.col('Year').alias('Year_1'),
        f.col('reported_imports_sum').alias('reported_imports_sum_1'),
        f.col('reported_exports_sum').alias('reported_exports_sum_1')
    )

temp_table_2 = country_reporting_indicators_data \
    .where((f.col('NomenCode') == NomenCode) & (f.col('flow_type')=='export_reporting') & (f.col('Year')<=2021)) \
    .select(
        f.col('Country').alias('Country_2'),
        f.col('Year').alias('Year_2'),
        f.col('reported_imports_sum').alias('reported_imports_sum_2'),
        f.col('reported_exports_sum').alias('reported_exports_sum_2')
    )

join_condition = (
    (temp_table_1.Country_1==temp_table_2.Country_2) & (temp_table_1.Year_1==temp_table_2.Year_2)
)

result_df = temp_table_1.join(temp_table_2, on=join_condition, how='outer')
result_df= result_df.withColumn("trade_balance_fob", f.when( (f.col('Country_1').isNotNull()) & (f.col('Country_2').isNotNull()), f.col('reported_imports_sum_2')-f.col('reported_imports_sum_1') ).otherwise(f.lit(None))   )
result_df= result_df.withColumn("trade_balance_cif", f.when( (f.col('Country_1').isNotNull()) & (f.col('Country_2').isNotNull()), f.col('reported_exports_sum_2')-f.col('reported_exports_sum_1') ).otherwise(f.lit(None))   )



# List of countries to plot
countries = ['MEX', 'ARG', 'BHS', 'LTU']
country_names=['<b>Mexico<b>','<b>Argentina<b>','<b>The Bahamas<b>','<b>Lithuania<b>']
line_styles = ['dash', 'dot', 'dashdot', 'longdash']
marker_symbols = ['circle', 'square', 'diamond', 'x']


fig = make_subplots(rows=2, cols=2, subplot_titles=country_names)
fig.for_each_annotation(lambda a: a.update(font=dict(size=22)))

    

fob_color = 'rgba(31, 119, 180, 0.6)'
cif_color = 'rgba(205, 92, 92, 0.7)' 

for i, country in enumerate(countries, start=1):
    viz_df = result_df.where((f.col('Country_1') == country) & (f.col('Year_1') != 2022)).toPandas().sort_values('Year_1')
    
    # Multiply trade balances by 1000 (original recored values are in 1000$)
    viz_df['trade_balance_fob'] *= 1000
    viz_df['trade_balance_cif'] *= 1000
    
    # Determine the row and column for the current subplot
    row = (i - 1) // 2 + 1
    col = (i - 1) % 2 + 1

    # Add FOB trace
    fig.add_trace(go.Scatter(x=viz_df['Year_1'], y=viz_df['trade_balance_fob'],
                             mode='lines+markers', line=dict(color=fob_color),
            marker=dict(symbol=marker_symbols[0],size=7), name='Trade Balance FOB'),
                  row=row, col=col)
    
    # Add CIF trace
    fig.add_trace(go.Scatter(x=viz_df['Year_1'], y=viz_df['trade_balance_cif'],
                             mode='lines+markers', line=dict(color=cif_color),
            marker=dict(symbol=marker_symbols[1],size=7), name='Trade Balance CIF'),
                  row=row, col=col)


fig.update_traces(showlegend=False)
fig.data[0].showlegend = True
fig.data[1].showlegend = True

# Update layout
fig.update_layout(height=1200, width=1200,
                  plot_bgcolor='white', font=dict(size=20), legend=dict(font=dict(size=20),orientation='h', 
        xanchor='center',
        x=0.5, 
        y=1.1))


for row in range(1, 3):  
    for col in range(1, 3):  
        fig.update_xaxes(tickfont=dict(size=18), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray', nticks=10, row=row, col=col)
        fig.update_yaxes(tickfont=dict(size=19), showline=True, linewidth=2, linecolor='black', gridcolor='lightgray', nticks=10, row=row, col=col)


fig.update_layout(legend_title_text='')

fig.show()

# COMMAND ----------


